
// Test for reverse_emulation

#include "reverse_emulation.h"

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

// unsigned casts convert negative values to large positive values

typedef long timestamp_t;
static timestamp_t emulator_frame = 0;
const int max_buf = 60;
static timestamp_t buffer [max_buf]; // timestamps of each frame in buffer
static int last_state = -1;
static int buf_size = max_buf;

void emu_save_state( int n )
{
	assert( n >= 0 );
	last_state = n;
}

void emu_restore_state( int n )
{
	assert( (unsigned) n <= last_state );
	emulator_frame = n * buf_size;
}

void emu_run_frame( int n )
{
	assert( (unsigned) n < buf_size );
	buffer [n] = emulator_frame++;
}

void emu_run_frame_invisibly()
{
	emulator_frame++;
}

void print_buffer()
{
	int i;
	for ( i = 0; i < buf_size; i++ )
	{
		int frame = buffer [i];
		printf( "Buffer [%d] holds frame %d.%d\n",
				i, frame / buf_size, frame % buf_size );
	}
}

int random( int range )
{
	return (long) rand() * range / (RAND_MAX + 1L);
}

void test( int bs )
{
	timestamp_t frame = 0;
	int n;
	
	buf_size = bs;
	
	re_init( buf_size );
	last_state = -1;
	emulator_frame = 0;
	memset( buffer, 0, sizeof buffer );
	
	re_next_frame();
	
	// randomly move to next and previous frames
	for ( n = 50000; n--; )
	{
		// choose random direction and count
		int dir = random( 2 );
		int count = random( buf_size * 2 );
		//printf( "%s %d frames from frame %d\n", (dir ? "Fwd" : "Rev"), length, frame );
		
		while ( count-- )
		{
			// keep within reasonable range
			if ( frame < buf_size * 3 )
				dir = 1;
			if ( frame > 1000 )
				dir = -1;
			
			// advance
			frame += (dir ? 1 : -1);
			{
				int index = (dir ? re_next_frame() : re_prev_frame());
				
				// check that correct frame would be displayed
				timestamp_t buffer_frame = buffer [index];
				if ( buffer_frame != frame )
				{
					print_buffer();
					assert( 0 );
				}
			}
		}
	}
}

int main()
{
	test( 1 ); // ruthless
	test( 2 ); // edge-case
	test( 3 ); // just in case
	test( 10 );
	test( max_buf );
	
	// report success and ensure that assertion failure is reported
	assert( !"Test passed" );
	
	return 0;
}

